@@ -1,8 +1,8 @@
 /* ad.c - routines for dealing with attribute descriptions */
-/* $OpenLDAP: pkg/ldap/servers/slapd/ad.c,v 1.95.2.9 2009/07/27 20:19:18 quanah Exp $ */
+/* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2009 The OpenLDAP Foundation.
+ * Copyright 1998-2024 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -74,9 +74,11 @@ static int option_count = 1;
 
 static int msad_range_hack = 0;
 
+static int ad_count;
+
 static Attr_option *ad_find_option_definition( const char *opt, int optlen );
 
-static int ad_keystring(
+int ad_keystring(
 	struct berval *bv )
 {
 	ber_len_t i;
@@ -143,7 +145,7 @@ static char *strchrlen(
 {
 	const char *p;
 
-	for( p=beg; *p && p < end; p++ ) {
+	for( p=beg; p < end && *p; p++ ) {
 		if( *p == ch ) {
 			*len = p - beg;
 			return (char *) p;
@@ -217,6 +219,7 @@ int slap_bv2ad(
 	optn = bv->bv_val + bv->bv_len;
 
 	for( opt=options; opt != NULL; opt=next ) {
+		Attr_option *aopt;
 		int optlen;
 		opt++; 
 		next = strchrlen( opt, optn, ';', &optlen );
@@ -243,11 +246,11 @@ int slap_bv2ad(
 			desc.ad_flags |= SLAP_DESC_BINARY;
 			continue;
 
-		} else if ( ad_find_option_definition( opt, optlen ) ) {
+		} else if (( aopt = ad_find_option_definition( opt, optlen )) ) {
 			int i;
 
 			if( opt[optlen-1] == '-' ||
-				( opt[optlen-1] == '=' && msad_range_hack )) {
+				( aopt->name.bv_val[aopt->name.bv_len-1] == '=' && msad_range_hack )) {
 				desc.ad_flags |= SLAP_DESC_TAG_RANGE;
 			}
 
@@ -269,6 +272,7 @@ int slap_bv2ad(
 
 				if( rc == 0 && (unsigned)optlen == tags[i].bv_len ) {
 					/* duplicate (ignore) */
+					ntags--;
 					goto done;
 
 				} else if ( rc > 0 ||
@@ -382,6 +386,9 @@ done:;
 		d2->ad_flags = desc.ad_flags;
 		d2->ad_cname.bv_len = desc.ad_type->sat_cname.bv_len;
 		d2->ad_tags.bv_len = desc.ad_tags.bv_len;
+		ldap_pvt_thread_mutex_lock( &ad_index_mutex );
+		d2->ad_index = ++ad_count;
+		ldap_pvt_thread_mutex_unlock( &ad_index_mutex );
 
 		if (dlen == 0) {
 			d2->ad_cname.bv_val = d2->ad_type->sat_cname.bv_val;
@@ -539,7 +546,7 @@ int ad_inlist(
 			}
 
 			/*
-			 * EXTENSION: if requested description is preceeded by
+			 * EXTENSION: if requested description is preceded by
 			 * a '-' character, do not match on subtypes.
 			 */
 			if ( attrs->an_name.bv_val[0] == '-' ) {
@@ -763,13 +770,15 @@ int slap_bv2undef_ad(
 
 		desc->ad_cname.bv_len = bv->bv_len;
 		desc->ad_cname.bv_val = (char *)(desc+1);
-		strcpy(desc->ad_cname.bv_val, bv->bv_val);
+		strncpy(desc->ad_cname.bv_val, bv->bv_val, bv->bv_len);
+		desc->ad_cname.bv_val[bv->bv_len] = '\0';
 
 		/* canonical to upper case */
 		ldap_pvt_str2upper( desc->ad_cname.bv_val );
 
 		/* shouldn't we protect this for concurrency? */
 		desc->ad_type = at;
+		desc->ad_index = 0;
 		ldap_pvt_thread_mutex_lock( &ad_undef_mutex );
 		desc->ad_next = desc->ad_type->sat_ad;
 		desc->ad_type->sat_ad = desc;
@@ -778,7 +787,7 @@ int slap_bv2undef_ad(
 		Debug( LDAP_DEBUG_ANY,
 			"%s attributeDescription \"%s\" inserted.\n",
 			( flags & SLAP_AD_PROXIED ) ? "PROXIED" : "UNKNOWN",
-			desc->ad_cname.bv_val, 0 );
+			desc->ad_cname.bv_val );
 	}
 
 	if( !*ad ) {
@@ -838,6 +847,9 @@ undef_promote(
 			tmp->ad_next = NULL;
 			/* ad_cname was contiguous, no leak here */
 			tmp->ad_cname = nat->sat_cname;
+			ldap_pvt_thread_mutex_lock( &ad_index_mutex );
+			tmp->ad_index = ++ad_count;
+			ldap_pvt_thread_mutex_unlock( &ad_index_mutex );
 			*n_ad = tmp;
 			n_ad = &tmp->ad_next;
 		} else {
@@ -1131,17 +1143,18 @@ file2anlist( AttributeName *an, const char *fname, const char *brkstr )
 
 	fp = fopen( fname, "r" );
 	if ( fp == NULL ) {
+		char ebuf[128];
+		int saved_errno = errno;
 		Debug( LDAP_DEBUG_ANY,
 			"get_attrs_from_file: failed to open attribute list file "
-			"\"%s\": %s\n", fname, strerror(errno), 0 );
+			"\"%s\": %s\n", fname, AC_STRERROR_R( saved_errno, ebuf, sizeof(ebuf) ) );
 		return NULL;
 	}
 
 	lcur = line = (char *) ch_malloc( lmax );
 	if ( !line ) {
 		Debug( LDAP_DEBUG_ANY,
-			"get_attrs_from_file: could not allocate memory\n",
-			0, 0, 0 );
+			"get_attrs_from_file: could not allocate memory\n" );
 		fclose(fp);
 		return NULL;
 	}
@@ -1160,8 +1173,7 @@ file2anlist( AttributeName *an, const char *fname, const char *brkstr )
 			line = (char *) ch_realloc( line, lmax );
 			if ( !line ) {
 				Debug( LDAP_DEBUG_ANY,
-					"get_attrs_from_file: could not allocate memory\n",
-					0, 0, 0 );
+					"get_attrs_from_file: could not allocate memory\n" );
 				fclose(fp);
 				return NULL;
 			}
