@@ -1,8 +1,8 @@
 #! /bin/sh
-# $OpenLDAP: pkg/ldap/tests/scripts/monitor_data.sh,v 1.2.2.2 2009/01/22 00:01:18 kurt Exp $
+# $OpenLDAP$
 ## This work is part of OpenLDAP Software <http://www.openldap.org/>.
 ##
-## Copyright 1998-2009 The OpenLDAP Foundation.
+## Copyright 1998-2024 The OpenLDAP Foundation.
 ## All rights reserved.
 ##
 ## Redistribution and use in source and binary forms, with or without
@@ -13,21 +13,18 @@
 ## top-level directory of the distribution or, alternatively, at
 ## <http://www.OpenLDAP.org/license.html>.
 
-MONITORDB="$1"
-SRCDIR="$2"
-DSTDIR="$3"
+SRCDIR="$1"
+DSTDIR="$2"
 
-echo "MONITORDB $MONITORDB"
 echo "SRCDIR $SRCDIR"
 echo "DSTDIR $DSTDIR"
 echo "pwd `pwd`"
 
 # copy test data
 cp "$SRCDIR"/do_* "$DSTDIR"
-if test $MONITORDB != no ; then
 
-	# add back-monitor testing data
-	cat >> "$DSTDIR/do_search.0" << EOF
+# add back-monitor testing data
+cat >> "$DSTDIR/do_search.0" << EOF
 cn=Monitor
 (objectClass=*)
 cn=Monitor
@@ -38,11 +35,8 @@ cn=Monitor
 (objectClass=*)
 EOF
 
-	cat >> "$DSTDIR/do_read.0" << EOF
+cat >> "$DSTDIR/do_read.0" << EOF
 cn=Backend 1,cn=Backends,cn=Monitor
 cn=Entries,cn=Statistics,cn=Monitor
 cn=Database 1,cn=Databases,cn=Monitor
 EOF
-
-fi
-
